@@ -1,9 +1,13 @@
 #ifndef MSTATDATA
 #define MSTATDATA
 
+#include "SpecialArrays.h"
+
 namespace extendedleaps {
 
-class symtwodarray;  /* forward declaration     */
+class symtwodarray; 
+class partialsqfdata; 
+class sqfdata; 
 
 class partialwilksdata :  public partialdata {     /* Data used in Wilks statistic updates    */
 	public:
@@ -27,12 +31,13 @@ class wilksdata :  public subsetdata {
 	public:
 		wilksdata(vind nv,vind tnv,vind nvtopiv,vind hr,real wst);
 		virtual ~wilksdata(void);
+		virtual bool max(void)  { return false; }
 		virtual const real criterion(void) const { return wilksst; }
 		virtual void setcriterion(real w)        { wilksst = w; }	
 		virtual const real indice(void)	const; 
 		virtual void  getpdata(partialdata *pd);  
-		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt) const;
-		virtual void pivot(direction,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last);
+		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt,bool& reliable,const double tol,const double) const;
+		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last,bool& reliable,const double tol);
 /* 
 	Note: partialdata and subsetdata pointer must point to partialwilksdata and wilksdata classes
 		  or unpredictable behaviour will result  
@@ -44,10 +49,15 @@ class wilksdata :  public subsetdata {
 		void setematcoef(vind i,vind j,real val)   { (*emat)(i,j) = val;  }
 		void settmatcoef(vind i,vind j,real val)   { (*tmat)(i,j) = val;  }
 		virtual void setorgvarl(vind *) {  }
+		virtual bool nopivot(void) const    { return unreliable; }
+		virtual void forbidpivot(void)	{ unreliable = true; }	
+		virtual void allowpivot(void)   { unreliable = false; }	
 	private:
-		real updatecrt(direction dir,vind varind,partialdata* newdtpnt) const;   
-		void pivot(lagindex<d>& prtmmit,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last);
-		void pivot(lagindex<i>& prtmmit,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last);
+		real updatecrt(direction dir,vind varind,partialdata* newdtpnt,bool& reliable,const double tol) const; 
+		void pivot(lagindex<d>& prtmmit,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last,bool& reliable,const double tol);
+		void pivot(lagindex<i>& prtmmit,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last,bool& reliable,const double tol);
+		bool nopivot(lagindex<d>& prtmmit,vind vp) const;
+		bool nopivot(lagindex<i>& prtmmit,vind vp) const;
 		vind		nvar;
 		vind		p;
 		vind		k;
@@ -55,12 +65,13 @@ class wilksdata :  public subsetdata {
 		real		wilksst;
 		symtwodarray*	emat;
 		symtwodarray*	tmat;
+		bool		unreliable;
 };
 
 class partialtracedata :  public partialdata {     /* Data used in trace statistic updates	*/
 	public:
 		partialtracedata(vind nvars,vind hrank);
-		virtual ~partialtracedata(void)			{ delete pqf;  }
+		virtual ~partialtracedata(void);
 		virtual const real	getcrt(void)	const;
 		partialsqfdata*  getpqfdata(void)	const	{ return pqf; }
 	protected:
@@ -72,19 +83,23 @@ class partialtracedata :  public partialdata {     /* Data used in trace statist
 class tracedata :  public subsetdata {
 	public:
 		tracedata(vind nv,vind tnv,vind nvtopiv,vind hr,real crt);
-		virtual ~tracedata(void) { delete sqf; }
+		virtual ~tracedata(void);
 		sqfdata*  getqfdata(void) const	{ return sqf; };
+		virtual bool max(void)  { return true; }
 		virtual const real criterion(void) const;
 		virtual void setcriterion(real c);	
 		virtual void setorgvarl(vind *) {  }
 		virtual void  getpdata(partialdata *);  
-		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt) const;
-		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last);
+		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt,bool& reliable,const double tol,const double) const;
+		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last,bool& reliable,const double tol);
 /* 
 	Note: partialdata and subsetdata pointer must point to partialtracedata and tracedata classes
 		  or unpredictable behaviour will result  
 		  (general partialdata and subsetdata classes were used in order to garantee upward compability)
 */
+		virtual bool nopivot(void) const;
+		virtual void forbidpivot(void);
+		virtual void allowpivot(void);
 		virtual const real*	getbnds(void)	const	{ return 0; }	
 	protected:
 		vind		hrank;
