@@ -20,7 +20,7 @@ class rvgdata : public globaldata {
 		real			gets2(vind i,vind j) const	{ return (*s2)(i,j); }
 	private:
 		vind			p;  
-		symtwodarray*	s2;
+		symtwodarray*		s2;
 		real			trs2_;
 };
 
@@ -52,11 +52,12 @@ class rvdata :  public subsetdata {
 			const deque<bool>& active,vind* origvarlist,real criterion);
 		virtual ~rvdata(void);
 		virtual void  getpdata(partialdata* pd);  
+		virtual bool max(void)  { return true; }
 		virtual const real criterion(void)	const	{ return crt;  }
 		virtual void setcriterion(real c)		{ crt = c; }
 		virtual const real indice(void)		const	{ return sqrt(crt/gdt->trs2()); } 
-		virtual real updatecrt(direction dir,mindices& mmind,vind var,partialdata* pdt) const;
-		virtual void pivot(direction dir,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last);
+		virtual real updatecrt(direction d,mindices& mmind,vind var,partialdata* pdt,bool& reliable,const double tol,const double) const;
+		virtual void pivot(direction d,mindices& mmind,vind vp,vind t,partialdata* pdt,subsetdata* fdt,bool last,bool& reliable,const double tol);
 /*
 	Note: subsetdata pointer must point to rvgdata class or unpredictable behaviour will result 
 	(general subsetdata class was used in order to garantee upward compability)
@@ -70,13 +71,22 @@ class rvdata :  public subsetdata {
 		rvgdata *getgdata(void) const			{ return gdt; }
 		void  sets2m1(vind i,vind j,real val)		{ s2m1[i][j] = val; }
 		real  gets2m1(vind i,vind j) const		{ return s2m1[i][j]; }
+		virtual bool nopivot(void) const	{ return unreliable; }	
+		virtual void forbidpivot(void)	{ unreliable = true; }
+		virtual void allowpivot(void)   { unreliable = false; }	
 	private:
-		real updatecrt(direction dir,lagindex<d>& prtmmit,itindex<d>& fmmind,vind var,partialdata* newdtpnt) const;   
-		real updatecrt(direction dir,lagindex<i>& prtmmit,itindex<i>& fmmind,vind var,partialdata* newdtpnt) const;   
-		void pivot(direction dir,lagindex<d>& prtmmit,itindex<d>& fmmind,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last);
-		void pivot(direction dir,lagindex<i>& prtmmit,itindex<i>& fmmind,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,bool last);
-		void cmpts2sm1(lagindex<d>&,itindex<d>&,partialrvdata* pdata,twodarray& outmat,vind* orgvlst,vind vp,bool* rowlst,bool* collst) const;
-		void cmpts2sm1(lagindex<i>& prtmmit,itindex<i>& fmmind,partialrvdata* pdata,twodarray& outmat,vind* orgvlst,vind vp,bool* rowlst,bool* collst) const;
+		real updatecrt(direction dir,lagindex<d>& prtmmit,itindex<d>& fmmind,vind var,partialdata* newdtpnt,bool& reliable,const double tol) const;
+		real updatecrt(direction dir,lagindex<i>& prtmmit,itindex<i>& fmmind,vind var,partialdata* newdtpnt,bool& reliable,const double tol) const;
+		void pivot(direction dir,lagindex<d>& prtmmit,itindex<d>& fmmind,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,
+				bool last,bool& reliable,const double tol);
+		void pivot(direction dir,lagindex<i>& prtmmit,itindex<i>& fmmind,vind vp,vind t,partialdata* newpdtpnt,subsetdata* newfdtpnt,
+				bool last,bool& reliable,const double tol);
+		bool nopivot(lagindex<d>& prtmmit,vind vp) const;
+		bool nopivot(lagindex<i>& prtmmit,vind vp) const;
+		void cmpts2sm1(lagindex<d>&,itindex<d>&,partialrvdata* pdata,twodarray& outmat,vind* orgvlst,vind vp,bool* rowlst,bool* collst,
+				bool reorder) const;
+		void cmpts2sm1(lagindex<i>& prtmmit,itindex<i>& fmmind,partialrvdata* pdata,twodarray& outmat,vind* orgvlst,vind vp,bool* rowlst,
+				bool* collst,bool reorder) const;
 /*  Computation of the S2*S^1 matrix product for sub-matrices defined by row (rowlst) and column (collst) boolean lists  */
 		real frobenius(twodarray& m,bool *inlst) const;
 /* Computation of the Frobenius norm for the sub-matrix defined by the boolean list inlst  */
@@ -90,6 +100,8 @@ class rvdata :  public subsetdata {
 		vector<matvectarray *>	ivct;
 		twodarray		s2m1;
 		rvgdata*		gdt;
+		bool	unreliable;
+		real **rpl;  // Vector of pointers to reals whose accuracy will be monitered
 };
 
 }
